import requests
from typing import int, List, str

class MoleculeSearcher():
    def __init__(self,
                 api_key:str = "",
                 search_type:str = "morgan",
                 search_quality:str = "fast",
                 n_neighbors:int = 10,
                 compute_descriptors:bool = True,
                 compute_properties:bool = True,
                 filter_molecules:bool = True,
                 response_format:str = "json"):

        # Search type
        self.search_type = search_type

        # Search quality
        self.search_quality = search_quality

        # Number of neighbors
        self.n_neighbors = n_neighbors

        # Paste your API key here
        self.api_key = api_key

        # Compute descriptors
        self.compute_descriptors = compute_descriptors
        
        # Compute properties
        self.compute_properties = compute_properties

        # Filter molecules : 'No solvants' filter
        self.filter_molecules = filter_molecules

        # Response format 
        self.response_format = response_format


    # Parse response based on response format
    def parse_response(self,resp,response_format):
        if response_format=="json":
            return resp.json()

        else:
            return resp.content
    
    def download_csv(self,resp,
                     result_filename=None):
        # Download result file to local storage
        if result_filename is None:
            result_filename=resp.headers["content-disposition"].split()[-1].replace("filename=","")
            save_path="./"
            
        with open(save_path+result_filename,'wb') as new_file:
            new_file.write(resp.content)


    def search_single(self,query:str,
                      search_type:str = None,
                      n_neighbors:int = None
                      ):
        if search_type is None:
            search_type=self.search_type

        if n_neighbors is None:
            n_neighbors=self.n_neighbors

        # Perform the request
        resp=requests.get("https://api.cheese.themama.ai/molsearch",
                    {"search_input":query,
                    "search_type":search_type,
                    "n_neighbors":n_neighbors,
                    "search_quality":self.search_quality,
                    "descriptors":self.compute_descriptors,
                    "properties":self.compute_properties,
                    "filter_molecules":self.filter_molecules}
                    ,headers={'Authorization': f"Bearer {self.api_key}"
                    },
                    verify=False)
        
        return self.parse_response(resp,response_format="json")
    

    def search_list(self,
                    query:List[str],
                    search_type:str = None,
                    n_neighbors:int = None,
                    response_format:str = None
                      ):
        if search_type is None:
            search_type=self.search_type

        if n_neighbors is None:
            n_neighbors=self.n_neighbors
        
        if response_format is None:
            self.response_format = response_format
            
        # Perform the request
        resp=requests.get("https://api.cheese.themama.ai/molsearch_array",
                    {"search_input":query,
                    "search_type":search_type,
                    "n_neighbors":n_neighbors,
                    "search_quality":self.search_quality,
                    "descriptors":self.compute_descriptors,
                    "properties":self.compute_properties,
                    "filter_molecules":self.filter_molecules}
                    ,headers={'Authorization': f"Bearer {self.api_key}",
                              'Accept': f"application/{response_format}"
                    },
                    verify=False)

        return self.parse_response(resp,response_format)

    

    def search_file(self,filename:str,                    
                    search_type:str = None,
                    n_neighbors:int = None,
                    response_format:str = None):
        file=open(filename,"rb")

        if search_type is None:
            search_type=self.search_type

        if n_neighbors is None:
            n_neighbors=self.n_neighbors
        
        if response_format is None:
            self.response_format = response_format

        resp=requests.post("http://api.cheese.themama.ai/molsearch_file",
                                files={"search_input":file},
                                data={"search_type":self.search_type,
                                "n_neighbors":n_neighbors,
                                "search_quality":self.search_quality,
                                "descriptors":self.compute_descriptors,
                                "properties":self.compute_properties,
                                "filter_molecules":self.filter_molecules},
                                
                                headers={'Authorization': f"Bearer {self.api_key}",
                                            'Accept': f"application/{response_format}"
                                            },
                                verify=False)
        
        return self.parse_response(resp,response_format)


import requests
from typing import int, List, str

class PropertyPredictor():
    def __init__(self,
                 api_key:str = "",
                 compute_descriptors:bool = True,
                 compute_properties:bool = True,
                 response_format:str = "json"):

        # Paste your API key here
        self.api_key = api_key

        # Compute descriptors
        self.compute_descriptors = compute_descriptors
        
        # Compute properties
        self.compute_properties = compute_properties

        # Response format 
        self.response_format = response_format


    # Parse response based on response format
    def parse_response(self,resp,response_format):
        if response_format=="json":
            return resp.json()

        else:
            return resp.content
    
    def download_csv(self,resp,
                     result_filename=None):
        # Download result file to local storage
        if result_filename is None:
            result_filename=resp.headers["content-disposition"].split()[-1].replace("filename=","")
            save_path="./"
            
        with open(save_path+result_filename,'wb') as new_file:
            new_file.write(resp.content)


    def predict_single(self,query:str):
        # Perform the request
        resp=requests.get("https://api.cheese.themama.ai/predict",
                        {"search_input":query,
                        "descriptors":self.compute_descriptors,
                        "properties":self.compute_properties,
                        },
                        headers={'Authorization': f"Bearer {self.api_key}"
                                 },
                    verify=False)
        
        return self.parse_response(resp,response_format="json")
    

    def predict_list(self,
                    query:List[str],
                    response_format:str = None
                      ):
        
        if response_format is None:
            self.response_format = response_format
            
        # Perform the request
        resp=requests.get("https://api.cheese.themama.ai/predict_array",
                    {"search_input":query,
                    "descriptors":self.compute_descriptors,
                    "properties":self.compute_properties}
                    ,headers={'Authorization': f"Bearer {self.api_key}",
                              'Accept': f"application/{response_format}"
                    },
                    verify=False)

        return self.parse_response(resp,response_format)

    

    def predict_file(self,filename:str,
                    response_format:str = None):
        file=open(filename,"rb")
        
        if response_format is None:
            self.response_format = response_format

        resp=requests.post("http://api.cheese.themama.ai/predict_file",
                                files={"search_input":file},
                                data={"descriptors":self.compute_descriptors,
                                        "properties":self.compute_properties},
                                
                                headers={'Authorization': f"Bearer {self.api_key}",
                                            'Accept': f"application/{response_format}"
                                            },
                                verify=False)
        
        
        return self.parse_response(resp,response_format)


from setuptools import setup, find_packages
from warnings import warn
import sys

# ----------- python version check
if sys.version_info.major != 3 or sys.version_info.minor < 6:
    print(sys.version_info)
    raise SystemError('Module written for Python 3.6+.')

# -------------- fill docstring
import os

this_directory = os.path.abspath(os.path.dirname(__file__))
if os.path.exists(os.path.join(this_directory, 'README.md')):
    # there is no manifest.in file, so it could be missing
    with open(os.path.join(this_directory, 'README.md'), encoding='utf-8') as f:
        __doc__ = f.read()

description = f'An official Python3 module to query the cheese chemical space search server (https://cheese.themama.ai/)'

setup(
    name='cheese-pypi-api',
    version='1.0.0',
    python_requires='>=3.7',
    packages=find_packages(),
    install_requires=['requests'],
    url='https://github.com/the-mama-ai/cheese-python-api',
    license='MIT',
    author='The MAMA AI',
    author_email='',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Chemistry',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    description=description,
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown'
)